@@ -35,21 +35,22 @@ contrast = function(object, ...)
 #' @rdname contrast
 #' @param object An object of class \code{emmGrid}
 #' @param method Character value giving the root name of a contrast method (e.g.
-#'   \code{"pairwise"} -- see \link{emmc-functions}). Alternatively, a named
-#'   \code{list} of coefficients (for a contrast or linear function) that must
-#'   each conform to the number of results in each \code{by} group. In a
-#'   multi-factor situation, the factor levels are combined and treated like a
-#'   single factor.
-#' @param interaction Character vector or logical value. If this is specified,
+#'   \code{"pairwise"} -- see \link{emmc-functions}). Alternatively, a function
+#'   of the same form, or a named \code{list} of coefficients (for a contrast or
+#'   linear function) that must each conform to the number of results in each
+#'   \code{by} group. In a multi-factor situation, the factor levels are
+#'   combined and treated like a single factor.
+#' @param interaction Character vector, logical value, or list. If this is specified,
 #'   \code{method} is ignored. See the \dQuote{Interaction contrasts} section
 #'   below for details.
 #' @param by Character names of variable(s) to be used for ``by'' groups. The
 #'   contrasts or joint tests will be evaluated separately for each combination
 #'   of these variables. If \code{object} was created with by groups, those are
 #'   used unless overridden. Use \code{by = NULL} to use no by groups at all.
-#' @param offset Numeric vector of the same length as each \code{by} group.
-#'   These values are added to their respective linear estimates. (It is ignored
-#'   when \code{interaction} is specified.)
+#' @param offset,scale Numeric vectors of the same length as each \code{by} group.
+#'   The \code{scale} values, if supplied, multiply their respective linear estimates, and
+#'   any \code{offset} values are added. Scalar values are also allowed. 
+#'   (These arguments are ignored when \code{interaction} is specified.)
 #' @param name Character name to use to override the default label for contrasts
 #'   used in table headings or subsequent contrasts of the returned object.
 #' @param options If non-\code{NULL}, a named \code{list} of arguments to pass
@@ -87,16 +88,21 @@ contrast = function(object, ...)
 #'   "revpairwise")}.
 #'
 #' @section Interaction contrasts: When \code{interaction} is specified,
-#'   interaction contrasts are computed: Contrasts are generated for each factor
-#'   separately, one at a time; and these contrasts are applied to the object
-#'   (the first time around) or to the previous result (subsequently). (Any
-#'   factors specified in \code{by} are skipped.) The final result comprises
-#'   contrasts of contrasts, or, equivalently, products of contrasts for the
-#'   factors involved. Processing is done in the order of appearance in
-#'   \code{object@levels}. With \code{interaction = TRUE}, \code{method} (if
-#'   specified as character) is used for each contrast. If \code{interaction} is
-#'   a character vector, the elements specify the respective contrast method(s);
-#'   they are recycled as needed.
+#'   interaction contrasts are computed. Specifically contrasts are generated
+#'   for each factor separately, one at a time; and these contrasts are applied
+#'   to the object (the first time around) or to the previous result
+#'   (subsequently). (Any factors specified in \code{by} are skipped.) The final
+#'   result comprises contrasts of contrasts, or, equivalently, products of
+#'   contrasts for the factors involved. Any named elements of \code{interaction}
+#'   are assigned to contrast methods; others are assigned in order of
+#'   appearance in \code{object@levels}. The contrast factors in the resulting 
+#'   \code{emmGrid} object are ordered the same as in \code{interaction}.
+#'   
+#'   \code{interaction} may be a character vector or list of valid contrast
+#'   methods (as documented for the \code{method} argument). If the vector or
+#'   list is shorter than the number needed, it is recycled. Alternatively, if
+#'   the user specifies \code{contrast = TRUE}, the contrast specified in
+#'   \code{method} is used for all factors involved.
 #'   
 #' @section Simple contrasts:
 #'   \code{simple} is essentially the complement of \code{by}: When
@@ -149,17 +155,26 @@ contrast = function(object, ...)
 #' }
 #'
 #' # An interaction contrast for tension:wool
-#' tw.emm <- contrast(warp.emm, interaction = c("poly", "consec"), by = NULL)
+#' tw.emm <- contrast(warp.emm, interaction = c(tension = "poly", wool = "consec"), 
+#'                    by = NULL)
 #' tw.emm          # see the estimates
 #' coef(tw.emm)    # see the contrast coefficients
+#' 
+#' # Use of scale and offset
+#' #   an unusual use of the famous stack-loss data...
+#' mod <- lm(Water.Temp ~ poly(stack.loss, degree = 2), data = stackloss)
+#' (emm <- emmeans(mod, "stack.loss", at = list(stack.loss = 10 * (1:4))))
+#' # Convert results from Celsius to Fahrenheit:
+#' confint(contrast(emm, "identity", scale = 9/5, offset = 32))
+#' 
 contrast.emmGrid = function(object, method = "eff", interaction = FALSE, 
-                        by, offset = NULL, name = "contrast", 
+                        by, offset = NULL, scale = NULL, name = "contrast", 
                         options = get_emm_option("contrast"), 
                         type, adjust, simple, combine = FALSE, ratios = TRUE, ...) 
 {
     if(!missing(simple))
         return(.simcon(object, method = method, interaction = interaction,
-                      offset = offset, name = name, options = options,
+                      offset = offset, scale = scale, name = name, options = options,
                       type = type, simple = simple, combine = combine, 
                       adjust = adjust, ...))
     if(missing(by)) 
@@ -177,27 +192,43 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
     if (is.logical(interaction) && interaction)
         interaction = method
     if (!is.logical(interaction)) { # i.e., interaction is not FALSE
-        if (!is.character(interaction))
-            stop("interaction requires named contrast function(s)")
         if(missing(adjust))
             adjust = "none"
-        ### by = NULL why was this here before ???
         vars = names(object@levels)
         k = length(vars)
         if(!is.null(by)) {
             vars = c(setdiff(vars, by), by)
             k = k - length(by)
         }
-        interaction = rep(interaction, k)[1:k]
+        nms = names(interaction)
+        interaction = as.list(rep(interaction, k)[1:k])
+        names(interaction) = c(nms, rep("", k))[1:k]
+        nms = vars[1:k]
+        if (is.null(names(interaction)))
+            names(interaction) = nms
+        else {
+            unnamed = which(!(names(interaction) %in% nms))
+            names(interaction)[unnamed] = setdiff(nms, names(interaction))
+            nms = names(interaction)
+        }
+        
+        
+        # if (!is.character(interaction))
+        #     stop("interaction requires named contrast function(s)")
+        ### by = NULL why was this here before ???
         tcm = NULL
         for (i in k:1) {
-            nm = paste(vars[i], interaction[i], sep = "_")
-            object = contrast.emmGrid(object, interaction[i], by = vars[-i], name = nm, ...)
+            if (is.character(interaction[[i]]))
+                nm = paste(nms[i], interaction[[i]], sep = "_")
+            else
+                nm = paste(nms[i], "custom", sep = "_")
+            pos = which(vars == nms[i])
+            object = contrast.emmGrid(object, interaction[[i]], by = vars[-pos], name = nm, ...)
             if(is.null(tcm))
                 tcm = object@misc$con.coef
             else
                 tcm = object@misc$con.coef %*% tcm
-            vars[i] = nm
+            vars[pos] = nm
         }
         object = update(object, by = by, adjust = adjust, ..., silent = TRUE)
         object@misc$is.new.rg = NULL
@@ -251,7 +282,7 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
     }
     # case like in old lsmeans, contr = list
     else if (!is.function(method))
-        stop("'method' must be a function or the basename of an '.emmc' function")
+        stop("'method' must be a list, function, or the basename of an '.emmc' function")
     
     # Get the contrasts; this should be a data.frame
     cmat = method(levs, ...)
@@ -264,6 +295,13 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
     else if (nrow(cmat) != nrow(args))
         stop("Nonconforming number of contrast coefficients")
     tcmat = t(cmat)
+    if (!is.null(scale)) {
+        if (length(scale) %in% c(1, nrow(tcmat)))
+            tcmat = tcmat * scale
+        else
+            stop("'scale' length of ", length(scale), 
+                 " does not conform with ", nrow(tcmat), " contrasts", call. = FALSE)
+    }
     
     if (is.null(by)) {
         linfct = tcmat %*% linfct
@@ -368,7 +406,7 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
         else {
             misc$initMesg = c(misc$initMesg, 
                               paste("Note: contrasts are still on the", misc$orig.tran, "scale"))
-            message("Use 'contrast(regrid(object), ...)' to obtain contrasts on the response scale")
+            message("Note: Use 'contrast(regrid(object), ...)' to obtain contrasts of back-transformed estimates")
             misc$tran = misc$tran.mult = misc$tran.offset = NULL
         }
     }
