@@ -1,5 +1,5 @@
 ##############################################################################
-#    Copyright (c) 2012-2017 Russell V. Lenth                                #
+#    Copyright (c) 2012-2019 Russell V. Lenth                                #
 #                                                                            #
 #    This file is part of the emmeans package for R (*emmeans*)              #
 #                                                                            #
@@ -27,15 +27,15 @@
 ###   "adjust" is the default multiplicity adjustment (used if adjust="auto" in emmeans)
 
 #' Contrast families
-#' 
-#' Functions with an extension of \code{.emmc} provide for named contrast 
-#' families. One of the standard ones documented here may be used, or 
-#' the user may write such a function.
+#'
+#' Functions with an extension of \code{.emmc} provide for named contrast
+#' families. One of the standard ones documented here may be used, or the user
+#' may write such a function.
 #'
 #' Each standard contrast family has a default multiple-testing adjustment as
 #' noted below. These adjustments are often only approximate; for a more
-#' exacting adjustment, use the interfaces provided to
-#' \code{glht} in the \pkg{multcomp} package.
+#' exacting adjustment, use the interfaces provided to \code{glht} in the
+#' \pkg{multcomp} package.
 #'
 #' \code{pairwise.emmc}, \code{revpairwise.emmc}, and \code{tukey.emmc} generate
 #' contrasts for all pairwise comparisons among estimated marginal means at the
@@ -43,11 +43,11 @@
 #' For factor levels A, B, C, D, \code{pairwise.emmc} generates the comparisons
 #' A-B, A-C, A-D, B-C, B-D, and C-D, whereas \code{revpairwise.emmc} generates
 #' B-A, C-A, C-B, D-A, D-B, and D-C. \code{tukey.emmc} invokes
-#' \code{pairwise.emmc} or \code{revpairwise.emmc} depending on \code{reverse}. 
+#' \code{pairwise.emmc} or \code{revpairwise.emmc} depending on \code{reverse}.
 #' The default multiplicity adjustment method is \code{"tukey"}, which is only
 #' approximate when the standard errors differ.
 #'
-#' \code{poly.emmc} generates orthogonal polynomial contrasts, assuming 
+#' \code{poly.emmc} generates orthogonal polynomial contrasts, assuming
 #' equally-spaced factor levels. These are derived from the
 #' \code{\link[stats]{poly}} function, but an \emph{ad hoc} algorithm is used to
 #' scale them to integer coefficients that are (usually) the same as in
@@ -64,8 +64,8 @@
 #' \code{trt.vs.ctrl}. The default multiplicity adjustment method is
 #' \code{"dunnettx"}, a close approximation to the Dunnett adjustment.
 #' \emph{Note} in all of these functions, it is illegal to have any overlap
-#' between the \code{ref} levels and the \code{exclude} levels. If any
-#' is found, an error is thrown.
+#' between the \code{ref} levels and the \code{exclude} levels. If any is found,
+#' an error is thrown.
 #'
 #' \code{consec.emmc} and \code{mean_chg.emmc} are useful for contrasting
 #' treatments that occur in sequence. For a factor with levels A, B, C, D, E,
@@ -74,50 +74,54 @@
 #' (A+B)/2, and D - (A+B+C)/3. With \code{reverse = TRUE}, these differences go
 #' in the opposite direction.
 #'
-#' \code{eff.emmc} and \code{del.eff.emmc} generate contrasts that compare each 
+#' \code{eff.emmc} and \code{del.eff.emmc} generate contrasts that compare each
 #' level with the average over all levels (in \code{eff.emmc}) or over all other
-#' levels (in \code{del.eff.emmc}). These differ only in how they are scaled. 
+#' levels (in \code{del.eff.emmc}). These differ only in how they are scaled.
 #' For a set of k EMMs, \code{del.eff.emmc} gives weight 1 to one EMM and weight
-#' -1/(k-1) to the others, while \code{eff.emmc} gives weights (k-1)/k and -1/k 
-#' respectively, as in subtracting the overall EMM from each EMM. The default 
-#' multiplicity adjustment method is \code{"fdr"}. This is a Bonferroni-based 
+#' -1/(k-1) to the others, while \code{eff.emmc} gives weights (k-1)/k and -1/k
+#' respectively, as in subtracting the overall EMM from each EMM. The default
+#' multiplicity adjustment method is \code{"fdr"}. This is a Bonferroni-based
 #' method and is slightly conservative; see \code{\link[stats]{p.adjust}}.
 #'
+#' \code{identity.emmc} simply returns the identity matrix (as a data frame),
+#' minus any columns specified in \code{exclude}. It is potentially useful in
+#' cases where a contrast function must be specified, but none is desired.
+#'
 #' @rdname emmc-functions
-#' @aliases emmc-functions 
+#' @aliases emmc-functions
 #' @param levs Vector of factor levels
-#' @param exclude integer vector of indices, or character vector of levels to exclude
-#'   from consideration. These levels will receive weight 0 in all contrasts.
-#'   Character levels must exactly match elements of \code{levs}.
+#' @param exclude integer vector of indices, or character vector of levels to
+#'   exclude from consideration. These levels will receive weight 0 in all
+#'   contrasts. Character levels must exactly match elements of \code{levs}.
 #' @param include integer or character vector of levels to include (the
 #'   complement of \code{exclude}). An error will result if the user specifies
 #'   both \code{exclude} and \code{include}.
 #' @param ... Additional arguments, passed to related methods as appropriate
-#' 
+#'
 #' @return A data.frame, each column containing contrast coefficients for levs.
 #'   The "desc" attribute is used to label the results in emmeans, and the
 #'   "adjust" attribute gives the default adjustment method for multiplicity.
-#'   
+#'
 #' @note Caution is needed in cases where the user alters the ordering of
-#'   results (e.g., using the the \code{"[...]"} operator), because the contrasts
-#'   generated depend on the order of the levels provided. For example, suppose
-#'   \code{trt.vs.ctrl1} contrasts are applied to two \code{by} groups with
-#'   levels ordered (Ctrl, T1, T2) and (T1, T2, Ctrl) respectively, then the
-#'   contrasts generated will be for (T1 - Ctrl, T2 - Ctrl) in the first group and
-#'   (T2 - T1, Ctrl - T1) in the second group, because the first level in each group
-#'   is used as the reference level.
+#'   results (e.g., using the the \code{"[...]"} operator), because the
+#'   contrasts generated depend on the order of the levels provided. For
+#'   example, suppose \code{trt.vs.ctrl1} contrasts are applied to two \code{by}
+#'   groups with levels ordered (Ctrl, T1, T2) and (T1, T2, Ctrl) respectively,
+#'   then the contrasts generated will be for (T1 - Ctrl, T2 - Ctrl) in the
+#'   first group and (T2 - T1, Ctrl - T1) in the second group, because the first
+#'   level in each group is used as the reference level.
 #'
 #' @examples
 #' warp.lm <- lm(breaks ~ wool*tension, data = warpbreaks)
 #' warp.emm <- emmeans(warp.lm, ~ tension | wool)
 #' contrast(warp.emm, "poly")
 #' contrast(warp.emm, "trt.vs.ctrl", ref = "M")
-#' 
+#'
 #' # Compare only low and high tensions
 #' # Note pairs(emm, ...) calls contrast(emm, "pairwise", ...)
 #' pairs(warp.emm, exclude = 2)
 #' # (same results using exclude = "M" or include = c("L","H") or include = c(1,3))
-#' 
+#'
 #' ### Setting up a custom contrast function
 #' helmert.emmc <- function(levs, ...) {
 #'     M <- as.data.frame(contr.helmert(levs))
@@ -229,8 +233,8 @@ trt.vs.ctrl.emmc = function(levs, ref = 1, reverse = FALSE,
         stop("Reference levels are out of range")
     k = length(levs)
     cnm = ifelse(length(ref)==1, 
-        levs[ref], 
-        paste("avg(", paste(levs[ref], collapse=","), ")", sep=""))
+                 levs[ref], 
+                 paste("avg(", paste(levs[ref], collapse=","), ")", sep=""))
     templ = rep(0, length(levs))
     templ[ref] = -1 / length(ref)
     M = data.frame(levs=levs)
@@ -373,6 +377,21 @@ mean_chg.emmc = function(levs, reverse = FALSE, exclude = integer(0), include, .
     M
 }
 
+# Non-contrasts -- just pass thru, possibly excluding some levels
+#' @rdname emmc-functions
+identity.emmc = function(levs, exclude = integer(0), include, ...) {
+    exclude = .get.excl(levs, exclude, include)
+    k = length(levs) - length(exclude)
+    M = as.data.frame(diag(length(levs)))
+    names(M) = levs
+    if(length(exclude) > 0)
+        M = M[ , -exclude, drop = FALSE]
+    attr(M, "desc") = "Identity"
+    attr(M, "famSize") = k
+    attr(M, "adjust") = "none"
+    M
+}
+
 
 ### utility to translate character keys to index keys
 #' @export
